/* 
 * Copyright 2008 Maurice Castro. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *    1. Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *    2. Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials provided
 *        with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY Maurice Castro ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Maurice Castro OR OTHER CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef k8055_h
#define k8055_h 1

#include <sys/stdint.h>

/* the array in this structure is "read" from the card */
typedef struct input
{
	uint8_t b[8];
	int err;
} input_t;

/* the array in this structure is "written" to the card */
typedef struct output
{
	uint8_t b[8];
} output_t;

/* device description */
typedef struct k8055_dev k8055_dev_t;
struct k8055_dev
{
	int fd;		/* hid and control file descriptor endpoint 0 */
	int fd_ep1;	/* data file descriptor endpoint1 */
	int (*write_output)(k8055_dev_t *, output_t);
	input_t (*read_input)(k8055_dev_t *);
};

/* locate a card */
char *find_k8055(int card_no);

/* check the device */
/* -1 cannot open */
/* 0 probably ugen */
/* 1 uhid - limitted functionality */
int is_hid_dev(char *dev);

/* open the device */
k8055_dev_t *open_usbhid(char *dev);
k8055_dev_t *open_usbgen(char *dev);

/* close the device */
void close_usb(k8055_dev_t *d);

/* read data from device */
/* users of this function should check the .err value in the returned */
/* structure. .err will be set to -1 if an error occurred 0 otherwise */
input_t read_input(k8055_dev_t *d);

/* input decoding "functions" */
#define button1(in) (in.b[0] & 0x10)
#define button2(in) (in.b[0] & 0x20)
#define button3(in) (in.b[0] & 0x01)
#define button4(in) (in.b[0] & 0x40)
#define button5(in) (in.b[0] & 0x80)

#define analog1(in) (in.b[2])
#define analog2(in) (in.b[3])

uint16_t counter1(input_t in);
uint16_t counter2(input_t in);

/* output functions */
/* these functions return 0 on success and -1 on error */
int reset(k8055_dev_t *d);
int reset_counter1(k8055_dev_t *d);
int reset_counter2(k8055_dev_t *d);
int set_anadig(k8055_dev_t *d, uint8_t dig, uint8_t an1, uint8_t an2);
int set_debounce1(k8055_dev_t *d, long ms);
int set_debounce2(k8055_dev_t *d, long ms);
#endif
