/* 
 * Copyright 2008 Maurice Castro. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 
 *    1. Redistributions of source code must retain the above copyright
 *        notice, this list of conditions and the following disclaimer.
 *    2. Redistributions in binary form must reproduce the above
 *        copyright notice, this list of conditions and the following
 *        disclaimer in the documentation and/or other materials provided
 *        with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY Maurice Castro ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL Maurice Castro OR OTHER CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <strings.h>
#include <fcntl.h>
#include "err.h"
#include "k8055.h"

char *prognam = "";
int debug = 0;

int do_cmd(k8055_dev_t *crd_dev, char *buf, input_t in)
{
	char *bufp;
	char **ap;
	char *args[4];
	static uint8_t dig = 0;
	static uint8_t an1 = 0;
	static uint8_t an2 = 0;

	/* separate the string into up to 4 parts at white space */
	bufp = buf;
	for (ap = args; (*ap = strsep(&bufp, " \t\n")) != NULL;)
	{
		if (**ap != '\0')
			if (++ap >= &args[4])
				break;
	}
	if (!strcasecmp(args[0], "set"))
	{
		if (!strcasecmp(args[1], "analog"))
		{
			if (!strcasecmp(args[2], "1"))
			{
				an1 = strtoul(args[3],NULL,0);
				set_anadig(crd_dev, dig, an1, an2);
				printf("OK\n");
			}
			else if (!strcasecmp(args[2], "2"))
			{
				an2 = strtoul(args[3],NULL,0);
				set_anadig(crd_dev, dig, an1, an2);
				printf("OK\n");
			}
			else
			{
				printf("Bad Command\n");
			}
		}
		else if (!strcasecmp(args[1], "digital"))
		{
				dig = strtoul(args[2],NULL,0);
				set_anadig(crd_dev, dig, an1, an2);
				printf("OK\n");
		}
		else if (!strcasecmp(args[1], "debounce"))
		{
			if (!strcasecmp(args[2], "1"))
			{
				set_debounce1(crd_dev, strtoul(args[3],NULL,0));
				printf("OK\n");
			}
			else if (!strcasecmp(args[2], "2"))
			{
				set_debounce2(crd_dev, strtoul(args[3],NULL,0));
				printf("OK\n");
			}
			else
			{
				printf("Bad Command\n");
			}
		}
		else
		{
			printf("Bad Command\n");
		}
	}
	else if (!strcasecmp(args[0], "get"))
	{
		if (!strcasecmp(args[1], "counter"))
		{
			if (!strcasecmp(args[2], "1"))
			{
				printf("%d\n", counter1(in));
			}
			else if (!strcasecmp(args[2], "2"))
			{
				printf("%d\n", counter2(in));
			}
			else if (!strcasecmp(args[2], "all"))
			{
				printf("%d %d\n", counter1(in), counter2(in));
			}
			else
			{
				printf("Bad Command\n");
			}
		}
		else if (!strcasecmp(args[1], "button"))
		{
			if (!strcasecmp(args[2], "1"))
			{
				printf("%d\n", button1(in));
			}
			else if (!strcasecmp(args[2], "2"))
			{
				printf("%d\n", button2(in));
			}
			else if (!strcasecmp(args[2], "3"))
			{
				printf("%d\n", button3(in));
			}
			else if (!strcasecmp(args[2], "4"))
			{
				printf("%d\n", button4(in));
			}
			else if (!strcasecmp(args[2], "5"))
			{
				printf("%d\n", button5(in));
			}
			else if (!strcasecmp(args[2], "all"))
			{
				printf("%d %d %d %d %d\n", button1(in), button2(in), button3(in), button4(in), button5(in));
			}
			else
			{
				printf("Bad Command\n");
			}
		}
		else if (!strcasecmp(args[1], "analog"))
		{
			if (!strcasecmp(args[2], "1"))
			{
				printf("%d\n", analog1(in));
			}
			else if (!strcasecmp(args[2], "2"))
			{
				printf("%d\n", analog2(in));
			}
			else if (!strcasecmp(args[2], "all"))
			{
				printf("%d %d\n", analog1(in), analog2(in));
			}
			else
			{
				printf("Bad Command\n");
			}
		}
		else if (!strcasecmp(args[1], "all"))
		{
			printf("%d %d %d %d %d", button1(in), button2(in), button3(in), button4(in), button5(in));
			printf(" %d %d", counter1(in), counter2(in));
			printf(" %d %d", analog1(in), analog2(in));
			printf("\n");
		}
		else
		{
			printf("Bad Command\n");
		}
	}
	else if (!strcasecmp(args[0], "reset"))
	{
		if (!strcasecmp(args[1], "counter"))
		{
			if (!strcasecmp(args[2], "1"))
			{
				reset_counter1(crd_dev);
				printf("OK\n");
			}
			else if (!strcasecmp(args[2], "2"))
			{
				reset_counter2(crd_dev);
				printf("OK\n");
			}
			else if (!strcasecmp(args[2], "all"))
			{
				reset_counter1(crd_dev);
				reset_counter2(crd_dev);
				printf("OK\n");
			}
			else
			{
				printf("Bad Command\n");
			}
		}
		else
		{
			printf("Bad Command\n");
		}
	}
	else if (!strcasecmp(args[0], "quit"))
	{
		printf("Quiting\n");
		return(1);
	}
	return(0);
}

void cmd_loop(k8055_dev_t *crd_dev)
{
	char buf[80];
	char *pt;
	char c;
	input_t in;
	int done = 0;
	int flags;
	/* make stdin non blocking */
	flags = fcntl(0,F_GETFL);
	if (flags < 0)
	{
		errexit("Unable to get flags for stdin", 0, 0);
	}
	flags |= O_NONBLOCK;
	if (fcntl(0,F_SETFL, flags) < 0)
	{
		errexit("Unable to make stdin nonblocking", 0, 0);
	}
	pt = buf;
	while (!done)
	{
		/* looks like the card must be polled regularly */
		in = read_input(crd_dev);
		/* poll stdin and execute when we have a newline */
		if (read(0, &c, 1) != -1)
		{
			if (c == '\n')
			{
				*pt = '\0';
				done = do_cmd(crd_dev, buf, in);
				pt = buf;
			}
			else if (pt < &buf[79])
			{
				*pt = c;
				pt++;
			}
		}
	}
	flags |= ~O_NONBLOCK;
	if (fcntl(0,F_SETFL, flags) < 0)
	{
		errexit("Unable to make stdin blocking", 0, 0);
	}
}

int main(int argc, char *argv[])
{
	k8055_dev_t *crd_dev = NULL;
	char *crd_devname;
	int crd_no = 0;

	/* initialise before error functions used */
	prognam = argv[0];

	if (argc < 2)
	{
		fprintf(stderr, "%s: Card address / number required\n", prognam);
		exit(1);
	}

	crd_no = strtoul(argv[1], NULL, 0);
	crd_devname = find_k8055(crd_no);

	if (!crd_devname)
	{
		fprintf(stderr, "%s: Card Number %d not found\n", prognam, crd_no);
		exit(1);
	}

	switch (is_hid_dev(crd_devname))
	{
		case -1:
			fprintf(stderr, "%s: Unable to open device %s\n", prognam, crd_devname);
			exit(1);
			break;
		case 1:
			fprintf(stderr, "%s: Warning: Card is attached to hid device %s: limitted functionality!\n", prognam, crd_devname);
			crd_dev = open_usbhid(crd_devname);
			break;
		case 0: 
			crd_dev = open_usbgen(crd_devname);
			break;
	}

	if (!crd_dev)
	{
		fprintf(stderr, "%s: Unable to open card %d\n", prognam, crd_no);
		exit(1);
	}

	reset(crd_dev);
	cmd_loop(crd_dev);

	close_usb(crd_dev);
	return(0);
}
